@@ -1,6 +1,7 @@
 from dm.ioc import RequiredFeature
 from dm.exceptions import *
 from dm.dictionarywords import VISITOR_NAME
+from dm.strategy import FindProtectionObjects
 
 class BaseAccessController(object):
     """Template class for controlling access to protected objects.
@@ -8,7 +9,9 @@ class BaseAccessController(object):
     Client objects will call the isAuthorised() method with keywords:
     
         accessController.isAuthorised(
-            person=joe, actionName='Update', protectedObject=joe
+            person=joe,
+            actionName='Update',
+            protectedObject=joe
         )
     
     Concrete access controllers inheriting this class simply involve
@@ -26,17 +29,17 @@ class BaseAccessController(object):
     """
     
     dictionary = RequiredFeature('SystemDictionary')
-    registry   = RequiredFeature('DomainRegistry')
-    logger     = RequiredFeature('Logger')
-    debug      = RequiredFeature('Debug')
+    registry = RequiredFeature('DomainRegistry')
+    logger = RequiredFeature('Logger')
+    debug = RequiredFeature('Debug')
 
     def __init__(self):
-        self.action           = None
-        self.person           = None
-        self.permissionObject = None
-        self.protectionObject = None
-        self.protectedObject  = None
-        self.visitor          = None
+        self.action = None
+        self.person = None
+        self.protectedObject = None
+        self.protectionObjects = None
+        self.permissions = None
+        self.visitor = None
         
     def isAuthorised(self, **kwds):
         singleUseInstance = self.__class__()
@@ -44,6 +47,7 @@ class BaseAccessController(object):
         return isAuthorised
 
     def isAccessAuthorised(self, person, actionName, protectedObject):
+        self.logAccessAttempt(person, actionName, protectedObject)
         try:
             self.setPerson(person)
             self.setAction(actionName)
@@ -51,56 +55,77 @@ class BaseAccessController(object):
             self.assertAccessAuthorised()
         except AccessIsAuthorised, inst:
             if self.debug:
+                if self.person:
+                    personName = self.person.name
+                else:
+                    personName = 'None'
                 self.logger.debug(
                     "Access Authorised: Person '%s' to '%s' object '%s'" % (
-                        self.person.name,
+                        personName,
                         self.actionName,
                         self.protectedObject,
                     )
                 )
             return True
         except AccessNotAuthorised, inst:
-            self.logger.info(
-                "Access Denied: Person '%s' to '%s' object '%s': %s" % (
-                    self.person.name, 
-                    self.actionName,
-                    self.protectedObject,
-                    inst,
+            if self.debug:
+                if self.person:
+                    personName = self.person.name
+                else:
+                    personName = 'None'
+                self.logger.debug(
+                    "Access Denied: Person '%s' to '%s' object '%s': %s" % (
+                        personName, 
+                        self.actionName,
+                        self.protectedObject,
+                        inst,
+                    )
                 )
-            )
         return False
 
+    def logAccessAttempt(self, person, actionName, protectedObject):
+        if self.debug:
+            if self.person:
+                personName = self.person.name
+            else:
+                personName = 'None'
+            self.logger.debug(
+                "Access Request: Person '%s' to '%s' object '%s'." % (
+                    personName, actionName, protectedObject,
+                )
+            )
+
     def setPerson(self, person):
+        self.permissions = None
         if not person:
             self.person = self.getVisitor()
         else:
             self.person = person
 
     def setAction(self, actionName):
+        self.permissions = None
         self.actionName = actionName
         if not actionName:
             raise AccessNotAuthorised("No action name.")
-        self.action = self.registry.actions[self.actionName]
+        if self.registry != None:
+            self.action = self.registry.actions[self.actionName]
+        else:
+            self.action = None
 
     def setProtectedObject(self, protectedObject):
+        self.permissions = None
         if not protectedObject:
             raise AccessNotAuthorised("No protected object.")
         self.protectedObject = protectedObject
-        self.makeProtectedNames()
-        self.setProtectionObject()
+        self.setProtectionObjects(protectedObject)
+        
+    def setProtectionObjects(self, protectedObject):
+        findObjects = FindProtectionObjects(protectedObject)
+        self.protectionObjects = findObjects.find()
 
     def assertAccessAuthorised(self):
         raise AccessNotAuthorised("Access not authorised, by default.")
 
-    def getPermissionObject(self):
-        if self.permissionObject == None:
-            permission = self.protectionObject.permissions[self.action]
-            self.permissionObject = permission
-        return self.permissionObject
-
-    def isPermissionGranted(self, grants):
-        return self.getPermissionObject() in grants
-
     def isRoleAuthorised(self, role):
         if self.isPermissionGranted(role.grants):
             if self.debug:
@@ -113,37 +138,32 @@ class BaseAccessController(object):
                 self.logger.debug(msg)
             return False
 
-    def makeProtectedNames(self):
-        self.protectedNames = []
-        if self.protectedObject.__class__ == type:
-            className = self.protectedObject.__name__
-            self.protectedNames.append(className)
-        else:
-            keyValue = self.protectedObject.getRegisterKeyValue()
-            className = self.protectedObject.__class__.__name__
-            instanceName = className + "." + str(keyValue)
-            self.protectedNames.append(instanceName)
-            self.protectedNames.append(className)
-        if not self.protectedNames:
-            raise "No protected names derived from protection object."
+    def isPermissionGranted(self, grants):
+        for permission in self.getPermissions():
+            if permission in grants:
+                return True
+        return False
 
-    def setProtectionObject(self):
-        protectionObjects = self.registry.protectionObjects
-        for name in self.protectedNames:
-            if name in protectionObjects:
-                self.protectionObject = protectionObjects[name]
-                return
-        raise "No protection object available for %s" % self.protectedNames
+    def getPermissions(self):
+        if self.permissions == None:
+            self.permissions = []
+            for protection in self.protectionObjects:
+                permission = protection.permissions[self.action]
+                self.permissions.append(permission)
+        return self.permissions
+
+    def isPersonNotVisitor(self):
+        return self.person != self.getVisitor()
 
     def getVisitor(self):
         if self.visitor == None:
-            visitorName = self.dictionary[VISITOR_NAME]
-            self.visitor = self.registry.persons[visitorName]
+            if self.registry != None:
+                visitorName = self.dictionary[VISITOR_NAME]
+                self.visitor = self.registry.persons[visitorName]
+            else:
+                return None
         return self.visitor
 
-    def isPersonNotVisitor(self):
-        return self.person.name != self.getVisitor().name
-
 
 class SystemAccessController(BaseAccessController):
     "Introduces system and personal roles to access controller."
@@ -225,14 +245,15 @@ class SystemAccessController(BaseAccessController):
  
     def isPersonAuthorisedOnSystem(self):
         systemRole = self.getPersonSystemRole()
+        roleName = getattr(systemRole, 'name', '')
         if self.isPermissionGranted(systemRole.grants):
             if self.debug:
-                msg = "Access authorised by person's system role."
+                msg = "Access authorised by '%s' system role." % roleName
                 self.logger.debug(msg)
             return True
         else:
             if self.debug:
-                msg = "Access not authorised by person's system role."
+                msg = "Access not authorised by '%s' system role." % roleName
                 self.logger.debug(msg)
             return False
 
