@@ -43,10 +43,13 @@ from dm.exceptions import KforgeDbError
 import sqlobject.main
 from dm.ioc import *
 from dictionarywords import *
+import mx.DateTime
 
 debug = RequiredFeature('Debug')
 logger = RequiredFeature('Logger')
 
+dbdebug = False
+
 class ConnectionFacade(object):
     "Presents database connection."
 
@@ -58,16 +61,18 @@ class ConnectionFacade(object):
         dbPass = self.dictionary[DB_PASS]
         dbHost = self.dictionary[DB_HOST]
         dbName = self.dictionary[DB_NAME]
-        uri = "%s://%s:%s@%s/%s" % (dbType, dbUser, dbPass, dbHost, dbName)
+        if self.dictionary.has_key(DB_URI):
+            uri = self.dictionary[DB_URI]
+        else:
+            uri = "%s://%s:%s@%s/%s" % (dbType, dbUser, dbPass, dbHost, dbName)
         try:
             self.connection = connectionForURI(uri)
             self.connection.makeConnection()
         except Exception, inst:
             msg = 'Could not connect to database: %s' % (inst)
             logger.critical(msg)
-            raise msg
-        if debug:
-            logger.debug('Connected to database OK.')
+            raise Exception, msg
+        logger.info('Connected to database OK.')
     
     def getConnection(self):
         return self.connection
@@ -99,21 +104,27 @@ class DatabaseFacade(object):
         def findRecord(self, className, *args, **kwds):
             "Retreive record of domain object."
             record = None
+            if dbdebug and debug:
+                logger.debug('Finding %s record in database.' % (className))
             if self.isSelectById(kwds) and len(kwds) == 1:
-                record = self.get(className, kwds['id'])
+                objectId = kwds['id']
+                try:
+                    record = self.get(className, objectId)
+                except SQLObjectNotFound, inst:
+                    msg = "Can't find %s record with id %s." \
+                        % (className, kwds)
+                    raise KforgeDbError, msg
             else:
                 records = self.findRecords(className, **kwds)
                 if hasattr(records, 'count'):
-                    if records.count():
+                    try:
                         record = records[0]
-                    else:
+                    except:
                         msg = "Can't find %s record with params %s %s." \
                             % (className, args, kwds)
                         raise KforgeDbError, msg
                 else:
                     record = records
-            if debug:
-                logger.debug('Found %s record from database.' % (className))
             return record
         
         def isSelectById(self, kwds):
@@ -130,12 +141,20 @@ class DatabaseFacade(object):
                 return True
             if '__endsBefore__' in kwds:
                 return True
+            if '__dateCreatedAfter__' in kwds:
+                return True
+            if '__dateCreatedBefore__' in kwds:
+                return True
+            if '__lastModifiedAfter__' in kwds:
+                return True
+            if '__lastModifiedBefore__' in kwds:
+                return True
             return False
 
         def get(self, className, id):
             "Retreive record of domain object."
             recordClass = self.getRecordClass(className)
-            return recordClass.get(id)
+            return recordClass.get(id)  # todo: pass transaction
 
         def findRecords(self, className, *args, **kwds):
             "Retreive records of domain objects."
@@ -167,14 +186,14 @@ class DatabaseFacade(object):
                 records = self.findRecords(className, *args, **kwds)
             else:
                 records = self.getRecordClass(className).select()
-            if debug:
+            if dbdebug and debug:
                 logger.debug('Listed %s records from database.' % (className))
             return records
        
         def createDomainObject(self, className, *args, **kwds):
             "Create new recorded domain object."
             newRecord = self.createRecord(className, *args, **kwds)
-            if debug:
+            if dbdebug and debug:
                 message = "Created new db record. %s" % newRecord
                 logger.debug(message)
             newObject = newRecord.getDomainObject()
@@ -196,11 +215,11 @@ class DatabaseFacade(object):
         def getRecordClass(self, className):
             "Returns record class for domain object type name."
             if not className: 
-                raise "Can't get record class without class name."
+                raise Exception, "Can't get record class without class name."
             try:
                 mapperClass = self.mappers[className]
             except Exception, inst:
-                raise "Mapper class '%s' not defined in: %s. (%s)" % (
+                raise Exception, "Mapper class '%s' not defined in: %s. (%s)" % (
                     className,
                     str(self.mappers),
                     str(inst)
@@ -219,7 +238,7 @@ class DatabaseFacade(object):
                 self.mappers[className] = mapperClass
                 self.checkMapperClassTable(className)
             elif not features.allowReplace:
-                raise "Mapper for '%s' already defined." % className
+                raise Exception, "Mapper for '%s' already defined." % className
 
         createMapperClass = classmethod(createMapperClass)
 
@@ -251,6 +270,8 @@ class DatabaseFacade(object):
             mapperClass = self.getRecordClass(className)
             mapperAttribute = mapperClass.meta.addAttribute(domAttribute)
             sqlAttribute = mapperAttribute.createNamedMapperClassAttribute()
+            if sqlAttribute == None:
+                return
             if hasattr(mapperClass.sqlmeta, 'addColumn'):   # for SQLObject 0.7
                 addColumn = mapperClass.sqlmeta.addColumn
                 delColumn = mapperClass.sqlmeta.delColumn
@@ -284,10 +305,14 @@ class DatabaseFacade(object):
         return setattr(self.__instance, attr, value)
 
 
+# todo: take reserved names from there pages:
+# http://developer.mimer.com/validator/parser200x/vendor-reserved-words-sql200x.tml
+
 class ReservedNames(object):
 
     reservedNames = None
     dictionary = RequiredFeature('SystemDictionary')
+
     sqlKeywords = {
         'common': [
             'ABSOLUTE', 'ADD', 'ADMIN', 'AFTER',
@@ -407,7 +432,7 @@ class MetaBase(object):
         elif hasattr(SQLObject, '_style'): # SQLObject 0.6
             return SQLObject._style
         else:
-            raise "Can't find SQLObject style object."
+            raise Exception, "Can't find SQLObject style object."
             
 
 class MetaMapper(MetaBase):
@@ -453,11 +478,29 @@ class MetaMapper(MetaBase):
             if self.dbName:
                 mapperAttributes['_table'] = self.dbName
         for a in self.attributes:
-            mapperAttributes[a.dbName] = a.createMapperClassAttribute()
+            mapperClassAttribute = a.createMapperClassAttribute()
+            if mapperClassAttribute:
+                mapperAttributes[a.dbName] = mapperClassAttribute
+            elif a.dom.isImageFile:
+                setMethodName = '_set_%s' % a.dbName
+                getMethodName = '_get_%s' % a.dbName
+                setMethod = a.dom.setFileContent
+                getMethod = a.dom.getFileContent
+                mapperAttributes[setMethodName] = setMethod
+                mapperAttributes[getMethodName] = getMethod
         return mapperAttributes
 
     def createClass(self, name, base, attrs):
-        return type(name, (base,), attrs)
+        newClass = type(name, (base,), attrs)
+        #for a in self.attributes:
+        #    if a.dom.isImageFile:
+        #        setMethodName = '_set_%s' % a.dbName
+        #        getMethodName = '_get_%s' % a.dbName
+        #        setMethod = a.dom.setFileContent
+        #        getMethod = a.dom.getFileContent
+        #        setattr(newClass, setMethodName, setMethod)
+        #        setattr(newClass, getMethodName, getMethod)
+        return newClass
 
 
 class MetaMapperAttribute(MetaBase):
@@ -502,13 +545,25 @@ class MetaMapperAttribute(MetaBase):
                 return IntCol(default=self.default)
             else:
                 return IntCol()
+        elif issubclass(self.dom.__class__, dm.dom.meta.BLOB):
+            if hasattr(self, 'default'):
+                return BLOBCol(default=self.default)
+            else:
+                return BLOBCol()
+        elif issubclass(self.dom.__class__, dm.dom.meta.Pickle):
+            if hasattr(self, 'default'):
+                return PickleCol(default=self.default)
+            else:
+                return PickleCol()
+        elif issubclass(self.dom.__class__, dm.dom.meta.ImageFile):
+            return None
         elif issubclass(self.dom.__class__, dm.dom.meta.DomainObjectRef):
             if hasattr(self, 'default'):
                 return ForeignKey(self.typeName, default=self.default)
             else:
                 return ForeignKey(self.typeName)
         else:
-            raise "Unknown domain attribute: %s" % self.dom
+            raise Exception, "Unknown domain attribute: %s" % self.dom
 
     def createNamedMapperClassAttribute(self):
         "Creates SQLObject columns for existing SQLObject classes."
@@ -532,6 +587,18 @@ class MetaMapperAttribute(MetaBase):
                 return IntCol(default=self.default)
             else:
                 return IntCol()
+        elif issubclass(self.dom.__class__, dm.dom.meta.BLOB):
+            if hasattr(self, 'default'):
+                return BLOBCol(default=self.default)
+            else:
+                return BLOBCol()
+        elif issubclass(self.dom.__class__, dm.dom.meta.Pickle):
+            if hasattr(self, 'default'):
+                return PickleCol(default=self.default)
+            else:
+                return PickleCol()
+        elif issubclass(self.dom.__class__, dm.dom.meta.ImageFile):
+            return None
         elif issubclass(self.dom.__class__, dm.dom.meta.DomainObjectRef):
             if hasattr(self, 'default'):
                 return KeyCol(foreignKey=self.typeName, name=self.dbName+'ID',
@@ -540,7 +607,7 @@ class MetaMapperAttribute(MetaBase):
             else:
                 return KeyCol(foreignKey=self.typeName, name=self.dbName+'ID')
         else:
-            raise "Unknown domain attribute: %s" % self.dom
+            raise Exception, "Unknown domain attribute: %s" % self.dom
 
 
 class Mapper(SQLObject):
@@ -552,6 +619,7 @@ class Mapper(SQLObject):
 
     domainObject = None
 
+
     def __init__(self, **kwds):
         self.coerceKwds(kwds)
         super(Mapper, self).__init__(**kwds)
@@ -596,32 +664,30 @@ class Mapper(SQLObject):
             obj = self.createDomainObject() 
             self.domainObject = obj 
             self.domainObject.record = self
-            if debug:
-                message = "Record returned newly instantiated %s." % self.getClassName()
-                logger.debug(message)
-            self.loadDomainObject(loadedList, sync=False)
+            self.loadDomainObject(loadedList, loadMapper=False)
         else:
             if not self.isCached():
                 if not self.domainObject in loadedList:
-                    if debug:
-                        message = "Record loads existing %s instance with values." % self.getClassName()
-                        logger.debug(message)
                     self.loadDomainObject(loadedList)
                 else:
-                    if debug:
-                        message = "Loaded %s record avoids loading loop." % self.getClassName()
+                    if dbdebug and debug:
+                        message = "Avoided circular loading of %s #%s." % (
+                            self.getClassName(), self.id
+                        )
                         logger.debug(message)
             else: 
-                if debug:
-                    message = "Record returned %s with existing values." % self.getClassName()
+                if dbdebug and debug:
+                    message = "Reusing %s #%s values." % (
+                        self.getClassName(), self.id
+                    )
                     logger.debug(message)
         return self.domainObject
 
     def createDomainObject(self):
-        "Creates recorded domain object."
+        "Reinstantiates pre-recorded domain object."
         domainClass = self.getDomainClass()
-        if debug:
-            message = "Instantiating domain object from: %s" % domainClass
+        if dbdebug and debug:
+            message = "Reinstantiating %s from record." % self.getClassName()
             logger.debug(message)
         domainObject = domainClass()
         return domainObject
@@ -635,35 +701,63 @@ class Mapper(SQLObject):
         registry = RequiredFeature('DomainRegistry')
         return registry.getDomainClass(className)
 
-    def loadDomainObject(self, loadedList, sync=True):
-        "Maps values from record to domain object."
-        if sync:
-            if debug:
-                message = "Synchronising mapper values with RDBMS."
-                logger.debug(message)
-            self.sync()
-        elif sync:
-            if debug:
-                message = "Not synchronising mapper values with RDBMS."
-                logger.debug(message)
-        if debug:
-            message = "Loading domain object values from mapper."
+    def loadDomainObject(self, loadedList, loadMapper=True):
+        loadedList[self.domainObject] = self.domainObject
+        if loadMapper:
+            self.loadMapperAttributesFromRecord()
+        self.loadDomainObjectFromMapperAttributes(loadedList)
+
+    def loadMapperAttributesFromRecord(self):
+        "Maps record values to mapper attributes."
+        if dbdebug and debug:
+            message = "Reloading mapper attributes from record."
+            logger.debug(message)
+        self.sync()
+
+    def loadDomainObjectFromMapperAttributes(self, loadedList):
+        if dbdebug and debug:
+            message = "Loading %s from mapper attributes..." % (
+                self.getClassName()
+            )
             logger.debug(message)
         self.domainObject.id = self.id
         for metaAttr in self.meta.attributes:
             dbName = metaAttr.dbName
             domName = metaAttr.domName
+            if dbdebug and debug:
+                message = "Loading %s %s attribute." % (
+                    self.getClassName(), domName,
+                )
+                logger.debug(message)
             if metaAttr.isDomainObjectRef:
                 mapper = getattr(self, dbName)
                 if mapper:
                     mappedValue = mapper.getDomainObject(loadedList)
                 else:
                     mappedValue = None
-                setattr(self.domainObject, domName, mappedValue)
+            elif metaAttr.dom.isValueRef and metaAttr.dom.isDateTime:
+                mappedValue = getattr(self, dbName)
+                if (mappedValue != None) and (mappedValue != ''):
+                    # Coerse to DateTime.
+                    # todo: stabilise SQLObject's date-time choice
+                    mappedValue = mx.DateTime.DateTime(
+                        mappedValue.year,
+                        mappedValue.month,
+                        mappedValue.day,
+                        mappedValue.hour,
+                        mappedValue.minute,
+                        mappedValue.second,
+                    )
+            elif metaAttr.dom.isImageFile:
+                mappedValue = getattr(self, dbName)
             else:
                 mappedValue = getattr(self, dbName)
-                setattr(self.domainObject, domName, mappedValue)
-        loadedList[self.domainObject] = self.domainObject
+            setattr(self.domainObject, domName, mappedValue)
+        if debug:
+            message = "Loaded %s from record #%s" % (
+                self.getClassName(), self.id,
+            )
+            logger.debug(message)
 
     def saveDomainObject(self):
         "Sets attributes of record object from domain object."
@@ -686,7 +780,7 @@ class Mapper(SQLObject):
                     setattr(self, metaAttr.dbName, domValue)
                     isChanged = True
         if isChanged:
-            if debug:
+            if dbdebug and debug:
                 message = "Updating RDBMS with %s mapper value." % (
                     self.meta.domName
                 )
@@ -694,7 +788,7 @@ class Mapper(SQLObject):
             self.syncUpdate()
 
     def coerceKwds(self, kwds):
-        "Translates incoming keyword parameters."
+        "Converting keyword names to database names."
         for metaAttr in self.meta.attributes:
             if metaAttr.domName in kwds:
                 if metaAttr.isDomainObjectRef:
@@ -840,6 +934,30 @@ class Mapper(SQLObject):
                     sqlSafeName, sqlSafeValue
                 )
                 sqlEqualsList.append(sqlEquals)
+            elif name == '__dateCreatedBefore__':
+                sqlSafeName = self.makeSqlName('date_created')
+                sqlEquals = "%s < %s" % (
+                    sqlSafeName, sqlSafeValue
+                )
+                sqlEqualsList.append(sqlEquals)
+            elif name == '__dateCreatedAfter__':
+                sqlSafeName = self.makeSqlName('date_created')
+                sqlEquals = "%s > %s" % (
+                    sqlSafeName, sqlSafeValue
+                )
+                sqlEqualsList.append(sqlEquals)
+            elif name == '__lastModifiedBefore__':
+                sqlSafeName = self.makeSqlName('last_modified')
+                sqlEquals = "%s < %s" % (
+                    sqlSafeName, sqlSafeValue
+                )
+                sqlEqualsList.append(sqlEquals)
+            elif name == '__lastModifiedAfter__':
+                sqlSafeName = self.makeSqlName('last_modified')
+                sqlEquals = "%s > %s" % (
+                    sqlSafeName, sqlSafeValue
+                )
+                sqlEqualsList.append(sqlEquals)
             else:
                 sqlSafeName = self.makeSqlName(name)
                 sqlEquals = "%s = %s" % (sqlSafeName, sqlSafeValue)
@@ -863,7 +981,9 @@ class Mapper(SQLObject):
         try:
             selection = self.selectBy(**kwds)
         except Exception, inst:
-            msg = "Couldn't execute selectBy() with %s: %s" % (kwds, inst)
+            msg = "Couldn't execute selectBy() on '%s' with %s: %s" % (
+                self.meta.domName, kwds, inst
+            )
             raise KforgeDbError, msg
         return selection
        
