@@ -1,9 +1,11 @@
 from dm.ioc import *
 from dm.dom.meta import *
-
 from dm.exceptions import *
+import dm.times
+from dm.util.datastructure import MultiValueDict
 
 debug = RequiredFeature('Debug')
+domdebug = False
 
 class DomainBase(object):
     """
@@ -70,16 +72,18 @@ class AbstractList(DomainBase):
         "Returns keyed object."
         if self.isCached:
             if key in self.cache:
-                if debug:
-                    message = 'Cache hit for key: %s' % key
+                if domdebug and debug:
+                    message = "Cache hit for key: '%s'" % key
                     self.log.debug(message)
                 return self.readCache(key)
             else:
-                if debug:
-                    message = 'Cache miss for key: %s' % key
+                if domdebug and debug:
+                    message = "Cache miss for key: '%s'" % key
                     self.log.debug(message)
-        if debug:
-            message = 'Retrieving record for key: %s' % key
+        if domdebug and debug:
+            message = "Retrieving from %s record for key: '%s'" % (self, key)
+            if message == 'WeekEarmarkTemplate':
+                raise Exception, message
             self.log.debug(message)
         return self.retrieveItem(key)
 
@@ -207,8 +211,11 @@ class DomainObjectRegister(AbstractList):
         results = self.database.startsWith(
             self.typeName, value, attributeName, **kwds
         )
-        iter = RegisterIterator(results=results, register=self)
-        return iter
+        #return RegisterIterator(results=results, register=self)
+        iterator = RegisterIterator(results=results, register=self)
+        objectList = [i for i in iterator]
+        self.sortDomainObjects(objectList)
+        return objectList
     
     def search(self, userQuery, attributeNames=None):
         kwds = {}
@@ -219,8 +226,10 @@ class DomainObjectRegister(AbstractList):
         results = self.database.search(
             self.typeName, userQuery, attributeNames, **kwds
         )
-        iter = RegisterIterator(results=results, register=self)
-        return iter
+        iterator = RegisterIterator(results=results, register=self)
+        objectList = [i for i in iterator]
+        self.sortDomainObjects(objectList)
+        return objectList
     
     def createDomainObject(self, *args, **kwds):
         try:
@@ -247,15 +256,37 @@ class DomainObjectRegister(AbstractList):
             record = self.findRecord(**kwds)
             return record.getDomainObject()
         else:
-            raise "No keyName set on %s" % self
+            raise Exception, "No keyName set on %s" % self
 
-    def read(self, *args, **kwds):
+    def read(self, *args, **kwds):  # todo: rename to something better
         self.coerceArgs(args, kwds)
         record = self.findRecord(**kwds)
         return record.getDomainObject()
             
+    def findSingleDomainObject(self, **kwds):
+        return self.findFirstDomainObject(**kwds)
+
+    def findFirstDomainObject(self, **kwds):
+        domainObjects = self.findDomainObjects(**kwds)
+        if len(domainObjects) >= 1:
+            return domainObjects[0]
+        else:
+            return None
+    
+    def findLastDomainObject(self, **kwds):
+        domainObjects = self.findDomainObjects(**kwds)
+        if len(domainObjects) >= 1:
+            return domainObjects[-1]
+        else:
+            return None
+    
     def findDomainObjects(self, **kwds):
-        return [r.getDomainObject() for r in self.findRecords(**kwds)]
+        objectList = []
+        for record in self.findRecords(**kwds):
+            domainObject = record.getDomainObject()
+            objectList.append(domainObject)
+        self.sortDomainObjects(objectList)
+        return objectList
     
     def findRecord(self, **kwds):
         "Finds existing object record."
@@ -313,10 +344,11 @@ class DomainObjectRegister(AbstractList):
             kwds[self.ownerName] = self.owner
         if self.owner2 and self.ownerName2:
             kwds[self.ownerName2] = self.owner2
-        if self.keyName and self.keyName in kwds:
-            keyValue = kwds[self.keyName]
-            if issubclass(keyValue.__class__, DomainObject):
-                kwds[self.keyName] = keyValue
+# Just speculating that the following lines are useless. - johnbywater
+#        if self.keyName and self.keyName in kwds:
+#            keyValue = kwds[self.keyName]
+#            if issubclass(keyValue.__class__, DomainObject):
+#                kwds[self.keyName] = keyValue
 
     def switchToRecords(self, kwds):
         metaAttributeNames = self.getDomainClassMetaAttributeNames()
@@ -332,7 +364,7 @@ class DomainObjectRegister(AbstractList):
                             msg = "No record for '%s' domainObjectRef: %s" % (
                                 attrName, kwds
                             )
-                            raise msg
+                            raise Exception, msg
                 
     def initialiseObject(self, object):
         object.initialise(self)
@@ -362,14 +394,13 @@ class DomainObjectRegister(AbstractList):
         attrName = self.getKeyName()
         return getattr(domainObject, attrName)
    
-    # todo: clear this up - aren't class attributes enough?
     def getKeyName(self):
         "Returns name of domain object attribute used for register key."
-        if hasattr(self, 'keyName'):
-            if self.keyName:  
-                return self.keyName
+        if hasattr(self, 'keyName') and self.keyName:
+            keyName = self.keyName
         else:
-            return 'id'
+            keyName = 'id'
+        return keyName
 
     def getStartsWithAttributeName(self):
         "Returns name of domain object attribute used in alphabetical index."
@@ -379,6 +410,62 @@ class DomainObjectRegister(AbstractList):
         "Returns name of attributes used for searching domain objects'."
         return self.searchAttributeNames or [self.getStartsWithAttributeName()]
 
+    def getObjectList(self):
+        objectList = [domainObject for domainObject in self]
+        return objectList
+    
+    def getSortedList(self):
+        objectList = self.getObjectList()
+        self.sortDomainObjects(objectList)
+        return objectList
+
+    def sortDomainObjects(self, domainObjects):
+        if debug:
+            msg = "Sorting domain object list (len %s)." % len(domainObjects)
+            self.log.debug(msg)
+        domainObjects.sort(self.cmpDomainObjects)
+
+    def cmpDomainObjects(self, x, y):
+        xValue = x.getSortOnValue()
+        yValue = y.getSortOnValue()
+        sortAscending = x.sortAscending
+        if xValue < yValue:
+            if sortAscending:
+                return -1
+            else:
+                return 1
+        if xValue == yValue:
+            return 0
+        if xValue > yValue:
+            if sortAscending:
+                return 1
+            else:
+                return -1
+
+    def getNextObject(self, objectList, domainObject):
+        if domainObject not in objectList:
+            return None
+            #msg = "Object not in list: %s %s" % (domainObject, objectList)
+            #raise Exception(msg)
+        index = objectList.index(domainObject)
+        index += 1
+        if (index < len(objectList)):
+            return objectList[index]
+        else:
+            return None
+
+    def getPreviousObject(self, objectList, domainObject):
+        if domainObject not in objectList:
+            return None
+            #msg = "Object not in list: %s %s" % (domainObject, objectList)
+            #raise Exception(msg)
+        index = objectList.index(domainObject)
+        index -= 1
+        if (index >= 0):
+            return objectList[index]
+        else:
+            return None
+
 
 class Register(DomainObjectRegister):
     pass
@@ -397,12 +484,16 @@ class DomainObject(DomainBase):
     isConstant = False
     isUnique = True
     searchAttributeNames = []
+    startsWithAttributeName = ''
     isRegistered = False
     dbName = ''
     registryAttrName = None
+    sortOnName = None 
+    sortAscending = True
 
     def __init__(self, **kwds):
         "Initialise base attributes."
+        #self.objectAttrValues = {}    # experimental
         self.assertIsClassRegistered()
         self.isChanged = False
         self.registerCache = {}
@@ -412,16 +503,45 @@ class DomainObject(DomainBase):
             for metaAttr in self.meta.attributes:
                 metaAttr.setInitialValue(self)
                 
+    ## experimental
+    #def __setattr__(self, attrName, attrVal):
+    #    if self.meta and self.meta.attributeNames.has_key(attrName) and hasattr(self, 'objectAttrValues'):
+    #        self.objectAttrValues[attrName] = attrVal
+    #    else:
+    #        super(DomainObject, self).__setattr__(attrName, attrVal)
+    #    #super(DomainObject, self).__setattr__(attrName, attrVal)
+   
+    ## experimental
+    #def __getattr__(self, attrName):
+    #    raise AttributeError(attrName)
+    #    if self.meta and self.meta.attributeNames.has_key(attrName) and hasattr(self, 'objectAttrValues'):
+    #        return self.objectAttrValues.get(attrName, None)
+    #    else:
+    #        raise AttributeError(attrName)
+
     def __repr__(self):
-        attrsRepr = "" 
+        className = self.__class__.__name__
+        reprAttrs = self.reprAttrs()
+        return "<%s %s>" % (className, reprAttrs)
+
+    def reprAttrs(self):
+        reprAttrs = ' id=%d' % self.id
         if self.meta:
             for attr in self.meta.attributes:
-                if attr.isValueRef:
-                    attrsRepr += " " + attr.createObjectRepr(self)
-        className = self.__class__.__name__
-        return "<%s id='%s'%s>" % (className, str(self.id), attrsRepr)
+                if attr.isValueRef and not attr.isBLOB:
+                    reprAttrs += ' %s' % attr.createObjectRepr(self)
+                elif attr.isDomainObjectRef:
+                    reprAttrs += ' %s' % attr.createObjectRepr(self)
+        return reprAttrs
 
-    def asNamedValues(self, excludeName=''):
+
+    def getSortOnValue(self):
+        if self.sortOnName:
+            return getattr(self, self.sortOnName)
+        else:
+            return self.getPersonalLabel()
+
+    def asNamedValues(self, excludeName=''):  # todo: excludeNames
         namedValues = [] 
         if self.meta:
             for attr in self.meta.attributes:
@@ -432,9 +552,15 @@ class DomainObject(DomainBase):
                     namedValue['label'] = attr.createLabelRepr(self)
                     namedValue['domainKeyValue'] = self.getRegisterKeyValue()
                     namedValue['isDomainObjectRef'] = attr.isDomainObjectRef
+                    if attr.isDomainObjectRef:
+                        namedValue['isSimpleOption'] = attr.isSimpleOption
+                    else:
+                        namedValue['isSimpleOption'] = False
                     namedValue['registryAttrName'] = attr.getRegistryAttrName(self)
                     namedValue['typeName'] = attr.typeName
                     namedValue['isAssociateList'] = attr.isAssociateList
+                    if attr.isAssociateList:
+                        namedValue['associatedNamedValues'] = attr.createAssociatedNamedValues(self)
                     namedValues.append(namedValue)
         return namedValues
 
@@ -445,6 +571,20 @@ class DomainObject(DomainBase):
                 dictValues[attr.name] = attr.createValueRepr(self)
         return dictValues
 
+    def asRequestParams(self):
+        requestParams = MultiValueDict()
+        for attr in self.meta.attributes:
+            attrName = attr.name
+            attrValueRepr = attr.createValueRepr(self)
+            if attr.isList():
+                requestParams.setlist(attrName, attrValueRepr)
+            else:
+                if attrValueRepr != None:
+                    requestParams[attrName] = attrValueRepr
+                else:
+                    requestParams[attrName] = ''
+        return requestParams
+
     def asDictLabels(self):
         labels = {} 
         if self.meta:
@@ -473,8 +613,17 @@ class DomainObject(DomainBase):
     
     def save(self):
         "Update record of object in the database."
-        self.record.saveDomainObject()
+        self.saveSilently()
         self.raiseUpdate()
+
+    def saveSilently(self):
+        "Update record without raising update event."
+        self.updateLastModifiedTime()
+        self.record.saveDomainObject()
+
+    def updateLastModifiedTime(self):
+        if 'lastModified' in self.meta.attributeNames:
+            self.lastModified = dm.times.getUniversalNow()
         
     def delete(self):
         self.deleteAggregates()
@@ -531,6 +680,12 @@ class DomainObject(DomainBase):
     def raiseUpdate(self):
         "Raises onUpdate event."
         self.onUpdate()
+        if debug:
+            message = "Updated %s: '%s'" % (
+                self.__class__.__name__,
+                self
+            )
+            self.log.debug(message)
 
     def raiseDelete(self):
         "Raises onDelete event."
@@ -541,6 +696,15 @@ class DomainObject(DomainBase):
         )
         self.log.info(message)
 
+    def raiseApprove(self):
+        "Raises onApprove event."
+        self.onApprove()
+        message = "Approved %s: '%s'" % (
+            self.__class__.__name__,
+            self.getRegisterKeyValue()
+        )
+        self.log.info(message)
+
     def raiseUndelete(self):
         "Raises onUndelete event."
         self.onUndelete()
@@ -551,7 +715,7 @@ class DomainObject(DomainBase):
         self.log.info(message)
 
     def raisePurge(self):
-        "Raises onDelete event."
+        "Raises onPurge event."
         self.onPurge()
 
     def onCreate(self):
@@ -566,6 +730,10 @@ class DomainObject(DomainBase):
         "Abstract handler for Delete object event."
         self.notifyPlugins(self.__class__.__name__ + 'Delete', self)
     
+    def onApprove(self):
+        "Abstract handler for Approve object event."
+        self.notifyPlugins(self.__class__.__name__ + 'Approve', self)
+    
     def onUndelete(self):
         "Abstract handler for Undelete object event."
         self.notifyPlugins(self.__class__.__name__ + 'Undelete', self)
@@ -590,9 +758,11 @@ class DomainObject(DomainBase):
             kwds['keyName']  = self.registerKeyName
         if not ('isCached' in kwds and kwds['isCached']):
             kwds['isCached'] = self.isConstant
-        if not ('searchAttributeNames' in kwds and kwds['searchAttributeNames']):
+        if not('searchAttributeNames' in kwds and kwds['searchAttributeNames']):
             kwds['searchAttributeNames'] = self.searchAttributeNames
-        if debug:
+        if not('startsWithAttributeName' in kwds and kwds['startsWithAttributeName']):
+            kwds['startsWithAttributeName'] = self.startsWithAttributeName
+        if domdebug and debug:
             className = self.registerClass.__name__
             message = "Creating '%s' register with kwds: %s" % (className, kwds)
             self.log.debug(message)
@@ -620,10 +790,12 @@ class DomainObject(DomainBase):
     
     # todo: rename to getLabel
     def getLabelValue(self):
+        # default to register key value
         return self.getRegisterKeyValue()
 
     def getPersonalLabel(self):
-        return self.getRegisterKeyValue()
+        # default to basic label
+        return self.getLabelValue()
 
     def createCollectiveNoun(self):
         noun = self.createNoun()
