@@ -6,6 +6,11 @@ from dm.ioc import *
 import md5
 import mx.DateTime
 from dm.datetimeconvertor import DateTimeConvertor
+from dm.datetimeconvertor import DateConvertor
+import Image
+from dm.dictionarywords import IMAGES_DIR_PATH
+from dm.dictionarywords import DB_MIGRATION_IN_PROGRESS
+import os
 
 class NotDefined(object):
     "Internal 'null' value."
@@ -15,8 +20,10 @@ class NotDefined(object):
 class MetaBase(object):
     "Domain model meta supertype."
 
-    registry   = RequiredFeature('DomainRegistry')
+    registry = RequiredFeature('DomainRegistry')
     dictionary = RequiredFeature('SystemDictionary')
+    logger = RequiredFeature('Logger')
+    debug = RequiredFeature('Debug')
 
     
 class MetaDomainObject(MetaBase):
@@ -42,7 +49,7 @@ class MetaDomainObject(MetaBase):
     def __setattr__(self, attrName, attrVal):
         if issubclass(attrVal.__class__, MetaDomainAttr):
             if attrName in self.reservedAttrNames:
-                raise "Domain attribute name '%s' is reserved." % attrName
+                raise Exception, "Domain attribute name '%s' is reserved." % attrName
             if not attrName in self.attributeNames:
                 attrVal.name = attrName
                 self.attributes.append(attrVal)
@@ -68,10 +75,11 @@ class MetaDomainAttr(MetaBase):
     isValueRef = False
     isDomainObjectRef = False
     isAssociateList = False
+    isImageFile = False
 
     def __init__(self, typeName='', name='', dbName='', default=NotDefined, title='', comment='', isEditable=True, isHidden=False, isRequired=True, getChoices=None, **kwds):
-        self.name = name
         self.typeName = typeName
+        self.name = name
         self.dbName = dbName or name
         if default != NotDefined: 
             self.default = default
@@ -118,17 +126,31 @@ class MetaDomainAttr(MetaBase):
         return "%s=\'%s\'" % (self.name, valueRepr)
 
     def createValueRepr(self, domainObject):
-        return getattr(domainObject, self.name)
+        value = getattr(domainObject, self.name)
+        return str(value)
 
     def createLabelRepr(self, domainObject):
         return self.createValueRepr(domainObject)
 
+    def createAssociatedNamedValues(self, domainObject):
+        raise Exception('No createAssociatedNamedValues method on %s' % self)
+
     def setAttributeFromMultiValueDict(self, domainObject, multiValueDict):
-        attrValue = self.makeValueFromMultiValueDict(multiValueDict)
-        self.setAttributeValue(domainObject, attrValue)
+        try:
+            attrValue = self.makeValueFromMultiValueDict(multiValueDict)
+        except Exception, inst:
+            msg = "Exception making '%s' value from multi-value dict: %s" % (
+                self.name, multiValueDict
+            )
+            self.logger.error(msg)
+            raise
+        if attrValue != self.getAttributeValue(domainObject):
+            msg = "Setting '%s' attribute to '%s'." % (self.name, attrValue)
+            self.logger.debug(msg)
+            self.setAttributeValue(domainObject, attrValue)
 
     def makeValueFromMultiValueDict(self, multiValueDict):
-        raise "Abstract method not implemented."
+        raise Exception, "Abstract method not implemented."
 
     def setAttributeValue(self, domainObject, attrValue):
         setattr(domainObject, self.name, attrValue)
@@ -149,20 +171,22 @@ class MetaDomainAttr(MetaBase):
             self.getAttributeValue(domainObject)
         )
 
-    def convertFromDbValue(dbValue):
+    def convertFromDbValue(self, dbValue):
         domainValue = dbValue
         return domainValue
         
-    def convertToDbValue(domainValue):
+    def convertToDbValue(self, domainValue):
         dbValue = domainValue
         return dbValue
-        
+
 
 class ValueObjectAttr(MetaDomainAttr):
     "Models a domain object value attribute."
 
     isValueRef = True
-    
+    isDateTime = False
+    isBLOB = False
+
     def __init__(self, **kwds):
         typeName = self.__class__.__name__
         super(ValueObjectAttr, self).__init__(typeName=typeName, **kwds)
@@ -207,14 +231,19 @@ class Password(String):
         super(Password, self).__init__(isRequired=isRequired, **kwds)
 
     def createValueRepr(self, domainObject):
-        return ""
+        if self.dictionary[DB_MIGRATION_IN_PROGRESS]:
+            return super(Password, self).createValueRepr(domainObject)
+        else:
+            return ""
 
     def makeValueFromMultiValueDict(self, multiValueDict):
-        passwordClear = multiValueDict[self.name]
-        if not passwordClear:
+        passwordText = multiValueDict[self.name]
+        if self.dictionary[DB_MIGRATION_IN_PROGRESS]:
+            return passwordText
+        elif passwordText:
+            return self.makeDigest(passwordText)
+        else:
             return ''
-        passwordDigest = self.makeDigest(passwordClear)
-        return passwordDigest
 
     def makeDigest(self, clearText):
         return md5.new(clearText).hexdigest()
@@ -225,11 +254,12 @@ class Password(String):
         
 
 class DateTime(ValueObjectAttr):
-    "Models a date-time concern."
+    "Models a date-time concern (no timezone)."
 
     convertor = DateTimeConvertor()
+    isDateTime = True
 
-    def __init__(self, default='', **kwds):
+    def __init__(self, default=None, **kwds):
         super(DateTime, self).__init__(default=default, **kwds)
 
     def makeValueFromMultiValueDict(self, multiValueDict):
@@ -241,17 +271,11 @@ class DateTime(ValueObjectAttr):
         dateTimeString = self.convertor.toHTML(dateTimeObject)
         return dateTimeString
             
-    #def convertFromDbValue(dbValue):
-    #    return self.convertor.fromHTML(dbValue)
-    #    
-    #def convertToDbValue(domainValue):
-    #    return self.convertor.toHTML(domainbValue)
-        
 
 class Date(DateTime):
     "Models a domain object date attribute."
     
-    pass
+    convertor = DateConvertor()
 
 
 class Boolean(ValueObjectAttr):
@@ -265,8 +289,16 @@ class Boolean(ValueObjectAttr):
         )
 
     def makeValueFromMultiValueDict(self, multiValueDict):
-        boolString = multiValueDict[self.name] or ''
-        return boolString.lower() in self.trueStrings
+        boolVal = multiValueDict[self.name] or False
+        if boolVal.__class__ == False.__class__:
+            return boolVal
+        elif boolVal.__class__ == ''.__class__:
+            return boolVal.lower() in self.trueStrings
+        else:
+            return False
+
+    def createValueRepr(self, domainObject):
+        return getattr(domainObject, self.name)
 
 
 class Integer(ValueObjectAttr):
@@ -279,6 +311,65 @@ class Integer(ValueObjectAttr):
         return int(multiValueDict[self.name])
 
 
+class BLOB(ValueObjectAttr):
+    "Models any binary large object attribute."
+
+    isBLOB = True
+
+    def makeValueFromMultiValueDict(self, multiValueDict):
+        return multiValueDict[self.name]
+
+
+class Pickle(BLOB):
+    "Models any Python object attribute."
+
+    pass
+    
+
+class ImageFile(MetaDomainAttr):
+    "Models an image attribute."
+    
+    isImageFile = True
+
+    def __init__(self, default='', maxWidth=None, **kwds):
+        super(ImageFile, self).__init__(default=default, typeName='ImageFile', **kwds)
+        self.maxWidth = maxWidth
+
+    def getFileContent(self, mapper):
+        filePath = self.getFilepath(mapper)
+        if filePath and os.path.exists(filePath):
+            file = open(filePath)
+            content = file.read()
+            file.close()
+            return content
+        else:
+            return None
+
+    def setFileContent(self, mapper, content):
+        if mapper:
+            filePath = self.getFilepath(mapper)
+            if filePath:
+                file = open(filePath, 'wb')
+                file.write(content)
+                file.close()
+            if self.maxWidth != None:
+                image = Image.open(filePath)
+                (width, height) = image.size
+                if width > self.maxWidth:
+                    widthToHeightRatio = float(width) / height
+                    newWidth = self.maxWidth
+                    newHeight = newWidth / widthToHeightRatio
+                    newSize = (newWidth, newHeight)
+                    newImage = image.resize(newSize, Image.ANTIALIAS)
+                    newImage.save(filePath)
+
+    def getFilepath(self, mapper):
+        # Extn .png doesn't break PIL Image.save() after resize; no extn does.
+        fileName = "%s%s.png" % (mapper.__class__.__name__, mapper.id)
+        dirPath = self.dictionary[IMAGES_DIR_PATH]
+        return os.path.join(dirPath, fileName)
+
+
 class DomainObjectAssociation(MetaDomainAttr):
     "Associates domain objects with other domain objects."
 
@@ -294,7 +385,7 @@ class DomainObjectAssociation(MetaDomainAttr):
         choices = []
         try:
             associateRegister = self.getAssociatedObjectRegister(domainObject)
-            for associateObject in associateRegister:
+            for associateObject in associateRegister.getSortedList():
                 key = associateObject.getRegisterKeyValue()
                 label = associateObject.getLabelValue()
                 choice = (key, label)
@@ -326,12 +417,11 @@ class DomainObjectRef(DomainObjectAssociation):
 
     isDomainObjectRef = True
 
-    def __init__(self, className='', **kwds):
+    def __init__(self, className='', isSimpleOption=False, **kwds):
+        self.isSimpleOption = isSimpleOption
         if not className:
             raise Exception, "No className for DomainObjectRef instance."
-        super(DomainObjectRef, self).__init__(
-            typeName=className, **kwds
-        )
+        super(DomainObjectRef, self).__init__(typeName=className, **kwds)
     
     def getAquaintance(self, domainObject):
         return getattr(domainObject, self.name)
@@ -404,9 +494,9 @@ class Composite(Composition, Aggregate):
 class AssociateList(DomainObjectAssociation):
     "Models aquaintance with several domain objects."
 
-    registerClass = None
     isAssociateList = True
     isPagedList = False
+    registerClass = None
 
     def __init__(self, className='', key='', owner='', isRequired=False, **kwds):
         super(AssociateList, self).__init__(typeName=className, isRequired=isRequired, **kwds)
@@ -423,7 +513,10 @@ class AssociateList(DomainObjectAssociation):
     def getAllChoices(self, domainObject):
         choices = []
         associateRegister = self.getAssociatedObjectRegister(domainObject)
-        for associateObject in associateRegister:
+        objectList = []
+        if associateRegister != []:
+            objectList = associateRegister.getSortedList()
+        for associateObject in objectList:
             key = associateRegister.getRegisterKey(associateObject)
             if self.isKeyDomainObject():
                 label = associateObject.getLabelValue()
@@ -446,8 +539,10 @@ class AssociateList(DomainObjectAssociation):
         return register
 
     def createValueRepr(self, domainObject):
-        register = self.getAssociationObjectRegister(domainObject)
         valuesList = []
+        register = self.getAssociationObjectRegister(domainObject)
+        if register == None:
+            return valuesList
         if self.isKeyDomainObject():
             valuesList = [i.getRegisterKeyValue() for i in register.keys()]
         else:
@@ -458,10 +553,10 @@ class AssociateList(DomainObjectAssociation):
         register = self.getAssociationObjectRegister(domainObject)
         labelsList = []
         if self.isKeyDomainObject():
-            labelsList = [str(i.getLabelValue()) for i in register.keys()]
+            labelsList = [str(register.getRegisterKey(i).getLabelValue()) for i in register.getSortedList()]
             labelsList = ", ".join(labelsList)
         else:
-            labelsList = [str(i) for i in register.keys()]
+            labelsList = [str(i.getLabelValue()) for i in register.getSortedList()]
             labelsList = ", ".join(labelsList)
         return labelsList or "_"
 
@@ -469,36 +564,63 @@ class AssociateList(DomainObjectAssociation):
         register = self.getAssociationObjectRegister(domainObject)
         valueLabels = []
         isKeyDomainObject = self.isKeyDomainObject()
-        for registerKey in register.keys():
+        for domainObject in register.getSortedList():
+            registerKey = register.getRegisterKey(domainObject)
             valueLabel = {}
             if isKeyDomainObject:
                 value = registerKey.getRegisterKeyValue() 
                 label = str(registerKey.getLabelValue())
             else:
                 value = registerKey
-                label = str(registerKey)
+                label = str(domainObject.getLabelValue())
             valueLabel['value'] = value
             valueLabel['label'] = label
             valueLabel['associationObject'] = register[registerKey]
             valueLabels.append(valueLabel)
         return valueLabels
 
+    def createAssociatedNamedValues(self, domainObject):
+        associatedNamedValues = []
+        associationRegister = self.getAssociationObjectRegister(domainObject)
+        if self.isKeyDomainObject():  # not bomb proof, really
+            for associatedObject in associationRegister.keys():
+                namedValue = {}
+                keyValue = associatedObject.getRegisterKeyValue() 
+                namedValue['key'] = keyValue
+                labelValue = str(associatedObject.getLabelValue())
+                namedValue['label'] = labelValue
+                namedValue['associatedObject'] = associatedObject
+                namedValue['associationObject'] = associationRegister[associatedObject]
+                associatedNamedValues.append(namedValue)
+        else:
+            for associatedObject in associationRegister:
+                namedValue = {}
+                keyValue = associatedObject.getRegisterKeyValue() 
+                namedValue['key'] = keyValue
+                labelValue = str(associatedObject.getLabelValue())
+                namedValue['label'] = labelValue
+                namedValue['associatedObject'] = associatedObject
+                namedValue['associationObject'] = associatedObject
+                associatedNamedValues.append(namedValue)
+        return associatedNamedValues
+
     def setAttributeFromMultiValueDict(self, domainObject, multiValueDict):
         associatedObjectKeys = multiValueDict.getlist(self.name)
-        associatedObjectRegister = self.getAssociatedObjectRegister(domainObject)
-        associationObjectRegister = self.getAssociationObjectRegister(domainObject)
+        associatedObjectRegister = self.getAssociatedObjectRegister(
+            domainObject)
+        associationObjectRegister = self.getAssociationObjectRegister(
+            domainObject)
         # delete where surplus
         for associationObject in associationObjectRegister:
             associatedObjectKey = getattr(associationObject, self.key)
             if self.isKeyDomainObject():
                 associatedObjectKey = associatedObjectKey.getRegisterKeyValue()
-            if not associatedObjectKey in associatedObjectKeys:
+            if not str(associatedObjectKey) in associatedObjectKeys:
                 associationObject.delete()
         # create where missing
         for associatedObjectKey in associatedObjectKeys:
             if associatedObjectKey == None:
                 continue
-                
             if self.isKeyDomainObject():
                 associatedObjectKey = associatedObjectRegister[associatedObjectKey]
             if not associatedObjectKey in associationObjectRegister:
@@ -515,7 +637,15 @@ class AssociateList(DomainObjectAssociation):
         
     def getKeyMetaAttribute(self):
         domainClass = self.getDomainClass()
-        return domainClass.meta.attributeNames[self.key]
+        if self.key in domainClass.meta.attributeNames:
+            return domainClass.meta.attributeNames[self.key]
+        else:
+            msg = "For class '%s', key '%s' not in attributes: %s." % (
+                domainClass.meta.name,
+                self.key,
+                domainClass.meta.attributeNames.keys(),
+            )
+            raise Exception(msg)
         
     def getAssociatedObjectRegister(self, domainObject):
         associateRegister = None
