@@ -3,6 +3,7 @@ import dm.testunit
 import dm.dom.metatest
 import dm.dom.accesscontroltest
 import dm.dom.persontest
+import dm.dom.imagetest
 
 def suite():
     "Return a TestSuite of dm.db TestCases."
@@ -10,6 +11,7 @@ def suite():
         dm.dom.metatest.suite(),
         dm.dom.accesscontroltest.suite(),
         dm.dom.persontest.suite(),
+        dm.dom.imagetest.suite(),
     ]
     return unittest.TestSuite(suites)
 
