@@ -23,3 +23,6 @@ class TestSystemEnvironment(unittest.TestCase):
         out = self.environment.getConfigFilePathEnvironmentVariableName()
         self.failUnlessEqual(exp, out)
 
+    def test_getPythonPath(self):
+        self.environment.getPythonPath()
+
