@@ -2,20 +2,22 @@
 
 # todo: Rename Kforge out of this module.
 
-class DmException(StandardError):
-    "System exception super class."
-    pass
+class DomainModelApplicationError(StandardError): pass
 
-class AccessControlException(DmException):
-    pass
+class InvalidSystemDictionary(DomainModelApplicationError): pass
 
-class AccessIsAuthorised(AccessControlException):
-    pass
+class AccessControlException(DomainModelApplicationError): pass
 
-class AccessNotAuthorised(AccessControlException):
-    pass
+class AccessIsAuthorised(AccessControlException): pass
 
-class KforgeError(StandardError):
+class AccessNotAuthorised(AccessControlException): pass
+
+class DataMigrationError(DomainModelApplicationError): pass
+
+class MissingPluginSystem(DomainModelApplicationError): pass
+
+# todo: Rework the KForge exception classes.
+class KforgeError(DomainModelApplicationError):
     "Kforge exception superclass."
     pass
 
