@@ -22,8 +22,8 @@ class PluginBase(object):
     
     def getRegister(self):
         if self.extendsDomainModel:
-            raise "Abstract method not implemented on class '%s'." % (
-                self.__class__.__name__
+            raise Exception, "Abstract method not implemented on '%s'." % (
+                self.__class__
             )
         else:
             return None
