@@ -1,4 +1,6 @@
 import unittest
+import dm.timestest
+import dm.strategytest
 import dm.environmenttest
 import dm.logtest
 import dm.debugtest
@@ -13,10 +15,14 @@ import dm.filesystemtest
 import dm.accesscontroltest
 import dm.applicationtest
 import dm.viewtest
+import dm.djangotest
+import dm.migratetest
 from dm.testunit import ApplicationTestSuite
 
 def suite():
     suites = [
+        dm.timestest.suite(),
+        dm.strategytest.suite(),
         dm.environmenttest.suite(),
         dm.logtest.suite(),
         dm.debugtest.suite(),
@@ -31,6 +37,8 @@ def suite():
         dm.accesscontroltest.suite(),
         dm.viewtest.suite(),
         dm.applicationtest.suite(),
+        dm.djangotest.suite(),
+        dm.migratetest.suite(),
     ]
     return ApplicationTestSuite(suites)
 
