@@ -5,11 +5,18 @@ from dm.dictionarywords import *
 from dm.view.manipulator import DomainObjectManipulator
 from dm.view.manipulator import HasManyManipulator
 from dm.view.manipulator import FormWrapper
-from django.core import template_loader
-from django.http import HttpResponse
-from django.http import HttpResponseRedirect
-from django.utils.datastructures import MultiValueDict
-from django.template import Context
+from dm.strategy import MakeCheckString
+from dm.strategy import MakeCookieString
+from dm.strategy import ValidateCookieString
+from dm.util.datastructure import MultiValueDict
+from dm.util.http import HttpResponse
+from dm.util.http import HttpResponseRedirect
+from dm.util.template import Context
+from dm.util.template import template_loader
+import dm.times
+import os
+
+moddebug = False
 
 class ControlledAccessView(object):
 
@@ -21,13 +28,13 @@ class ControlledAccessView(object):
     debug            = RequiredFeature('Debug')  # deprecated
     isDebug          = RequiredFeature('Debug')  # rename for debug
 
-    AUTH_SESSION_COOKIE_NAME = dictionary['auth_cookie_name']
-    NO_AUTH_SESSION_COOKIE_STRING = dictionary['no_auth_cookie_name']
-    if 'auth_cookie_domain' in dictionary:
-        AUTH_COOKIE_DOMAIN = dictionary['auth_cookie_domain']
+    authSessionCookieName = dictionary[AUTH_COOKIE_NAME]
+    noAuthSessionCookieString = dictionary[NO_AUTH_COOKIE_NAME]
+    if AUTH_COOKIE_DOMAIN in dictionary:
+        authCookieDomain = dictionary[AUTH_COOKIE_DOMAIN]
     else:
         # prefix with '.' to match subdomains
-        AUTH_COOKIE_DOMAIN = '.' + dictionary['domain_name']
+        authCookieDomain = '.' + dictionary[DOMAIN_NAME]
 
     def __init__(self, request=None, **kwds):
         self.request = request
@@ -41,7 +48,10 @@ class ControlledAccessView(object):
         self._canReadSystem = None
         self._canUpdateSystem = None
         self._canCreatePerson = None
+        self._canApprovePerson = None
+        self._canApprovePersons = None
         self._canReadPerson = None
+        self._canReadPersons = None
         self._canUpdatePerson = None
         self._canDeletePerson = None
         self._canCreateProject = None
@@ -87,19 +97,18 @@ class ControlledAccessView(object):
                     )
 
     def makeSessionKeyFromCookieString(self, cookieString):
-        sessionKey = cookieString[:32]
-        checkString = cookieString[32:]
-        if checkString == self.makeCheckString(sessionKey):
-            return sessionKey
-        else:
-            raise KforgeSessionCookieValueError(cookieString)
+        validateCookieStrategy = ValidateCookieString(cookieString)
+        return validateCookieStrategy.validate()
 
     def makeCookieStringFromSessionKey(self, sessionKey):
-        return sessionKey + self.makeCheckString(sessionKey) 
+        cookieStringStrategy = MakeCookieString(sessionKey)
+        cookieString = cookieStringStrategy.make()
+        return cookieString
 
     def makeCheckString(self, sessionKey):
-        secretKey = self.dictionary['django.secret_key']
-        return md5.new(sessionKey + secretKey + 'auth').hexdigest()
+        checkStringStrategy = MakeCheckString(sessionKey)
+        checkString = checkStingSrategy.make()
+        return checkString
 
     def findSession(self, sessionKey):
         if not sessionKey:
@@ -136,6 +145,9 @@ class ControlledAccessView(object):
     def canCreate(self, protectedObject):
         return self.authoriseActionObject('Create', protectedObject)
 
+    def canApprove(self, protectedObject):
+        return self.authoriseActionObject('Approve', protectedObject)
+
     def canRead(self, protectedObject):
         return self.authoriseActionObject('Read', protectedObject)
 
@@ -146,7 +158,10 @@ class ControlledAccessView(object):
         return self.authoriseActionObject('Delete', protectedObject)
 
     def getDomainClass(self, domainClassName):
-        return self.registry.getDomainClass(domainClassName)
+        if self.registry != None:
+            return self.registry.getDomainClass(domainClassName)
+        else:
+            return None
 
     def canReadSystem(self):
         if self._canReadSystem == None:
@@ -169,6 +184,27 @@ class ControlledAccessView(object):
             self._canCreatePerson = self.canCreate(protectedObject)
         return self._canCreatePerson
 
+    def canApprovePerson(self):
+        if self._canApprovePerson == None:
+            if self.person:
+                protectedObject = self.person
+            else:
+                protectedObject = self.getDomainClass('Person')
+            self._canApprovePerson = self.canApprove(protectedObject)
+        return self._canApprovePerson
+
+    def canApprovePersons(self):
+        if self._canApprovePersons == None:
+            protectedObject = self.getDomainClass('Person')
+            self._canApprovePersons = self.canApprove(protectedObject)
+        return self._canApprovePersons
+
+    def canReadPersons(self):
+        if self._canReadPersons == None:
+            protectedObject = self.getDomainClass('Person')
+            self._canReadPersons = self.canRead(protectedObject)
+        return self._canReadPersons
+
     def canReadPerson(self):
         if self._canReadPerson == None:
             if self.person:
@@ -327,7 +363,7 @@ class SessionView(ControlledAccessView):
         self.member = None
         self.service = None
         self.readRequest()
-        self.rememberme = None
+        self.requestParams = None
 
     def __repr__(self):
         return "<%s redirect='%s' redirectPath='%s'>" % (
@@ -355,13 +391,13 @@ class SessionView(ControlledAccessView):
 
     def readRequestSession(self):
         "Determines session from a cookie in request."
-        cookieName = self.AUTH_SESSION_COOKIE_NAME
+        cookieName = self.authSessionCookieName
         cookieString = self.request.COOKIES.get(cookieName, '')
         if not cookieString:
             if self.debug:
-                self.logger.debug('No KForge session cookie in request.')
+                self.logger.debug('No authentication session cookie in request.')
             return
-        if cookieString == self.NO_AUTH_SESSION_COOKIE_STRING:
+        if cookieString == self.noAuthSessionCookieString:
             if self.debug:
                 self.logger.debug('Unauthenticated session cookie in request.')
             return
@@ -370,12 +406,34 @@ class SessionView(ControlledAccessView):
         self.setSessionFromCookieString(cookieString)
         if self.session:
             self.session.updateLastVisited()
-            if self.debug:
+            if moddebug and self.debug:
                 self.logger.debug('Session re-established: %s' % self.session)
         else:
-            if self.debug:
+            if moddebug and self.debug:
                 self.logger.debug('No session for cookieString: %s' % cookieString)
         
+    def getRequestParam(self, paramName, default=None):
+        params = self.getRequestParams()
+        param = params.get(paramName, default)
+        return param
+
+    def getRequestParams(self):
+        if self.requestParams == None:
+            if self.request.POST:
+                self.requestParams = self.request.POST.copy()
+            elif self.request.GET:
+                self.requestParams = self.request.GET.copy()
+            else:
+                self.requestParams = self.getZeroRequestParams()
+            #if self.request.FILES:
+            #    self.requestParams.update(self.request.FILES)
+            if moddebug and self.debug:
+                self.logger.debug("Request params: %s" % self.requestParams)
+        return self.requestParams
+
+    def getZeroRequestParams(self):
+        return {}
+
     def getResponse(self):
         if self.checkAccessControl():
             self.logViewResponse(self.getMethodName(), 'OK')
@@ -472,9 +530,10 @@ class SessionView(ControlledAccessView):
     def renderContextWithTemplate(self):
         template = self.getTemplate()
         context = self.getContext()
-        self.logger.debug("Rendering template '%s' for context: %s" % (
-            self.templatePath, context
-        ))
+        if moddebug and self.debug:
+            self.logger.debug("Rendering template '%s' for context: %s" % (
+                self.templatePath, context
+            ))
         return template.render(context)
 
     def getTemplate(self):
@@ -483,19 +542,35 @@ class SessionView(ControlledAccessView):
         return self.template
 
     def loadTemplate(self):
+        templatesDir = self.dictionary[DJANGO_TEMPLATES_DIR]
+        if not os.path.exists(templatesDir):
+            msg = "Templates dir doesn't exist on path: %s" % templatesDir
+            raise Exception(msg)
         templatePath = self.getTemplatePath()
         templatePath += ".html"
-        self.template = template_loader.get_template(templatePath)
+        if moddebug and self.debug:
+            self.logger.debug("Loading template: %s" % templatePath)
+        try:
+            self.loadTemplateFromPath(templatePath)
+        except Exception, inst:
+            msg = "Failed to load template %s from %s: %s" % (
+                templatePath, templatesDir, inst
+            )
+            self.logger.error(msg)
+            raise Exception(msg)
  
     def getTemplatePath(self):
         if self.templatePath == None:
             self.templatePath = self.makeTemplatePath()
         if not self.templatePath:
-            raise "No templatePath set on %s" % self
+            raise Exception("No templatePath set on %s" % self)
         return self.templatePath
 
     def makeTemplatePath(self):
-        raise "No templatePath for view: %s" % self
+        raise Exception("No templatePath for view: %s" % self)
+
+    def loadTemplateFromPath(self, templatePath):
+        self.template = template_loader.get_template(templatePath)
 
     def getViewPosition(self):
         return self.getTemplatePath()
@@ -514,14 +589,17 @@ class SessionView(ControlledAccessView):
         else:
             systemServiceSocket = domainName
             
-        mediaHost = self.dictionary['www.media_host']
-        if self.dictionary['www.media_port'] != '80':
-            mediaPort = self.dictionary['www.media_port']
+        mediaHost = self.dictionary[MEDIA_HOST] 
+        if self.dictionary[MEDIA_PORT] != '80':
+            mediaPort = self.dictionary[MEDIA_PORT]
             systemMediaSocket = mediaHost +":"+ mediaPort
         else:
             systemMediaSocket = mediaHost
+       
+        pageCreated = dm.times.getLocalNowCWithZone()
         
         self.context.update({
+            'pageCreated'       : pageCreated,
             'view'              : self,
             'uriPrefix'         : self.dictionary[URI_PREFIX],
             'mediaPrefix'       : self.dictionary[MEDIA_PREFIX],
@@ -578,10 +656,10 @@ class SessionView(ControlledAccessView):
         if self.session:
             cookieString = self.makeCookieStringFromSessionKey(self.session.key)
         else:
-            cookieString = self.NO_AUTH_SESSION_COOKIE_STRING
-        cookieName = self.AUTH_SESSION_COOKIE_NAME
-        cookieDomain = self.AUTH_COOKIE_DOMAIN
-        if self.rememberme:
+            cookieString = self.noAuthSessionCookieString
+        cookieName = self.authSessionCookieName
+        cookieDomain = self.authCookieDomain
+        if self.getRequestParam('rememberme'):
             maxAge = 315360000
         else:
             maxAge = None
@@ -606,13 +684,13 @@ class SessionView(ControlledAccessView):
 class AbstractClassView(SessionView):
 
     registerSizeThreshold = 50
+    domainObjectKey = None
 
     def __init__(self, domainClassName=None, **kwds):
         super(AbstractClassView, self).__init__(**kwds)
         if domainClassName:
             self.domainClassName = domainClassName
         self.domainObject = None
-        self.domainObjectKey = None
 
     def getDomainObjectRegister(self):
         domainClass = self.getDomainClass(self.domainClassName)
@@ -632,17 +710,22 @@ class AbstractClassView(SessionView):
             if self.domainObjectKey:
                 objectRegister = self.getDomainObjectRegister()
                 if objectRegister.isStateful:
-                    self.domainObject = objectRegister.all[self.domainObjectKey]
+                    self.domainObject = objectRegister.getAll()[self.domainObjectKey]
                 else:
                     self.domainObject = objectRegister[self.domainObjectKey]
+            
         return self.domainObject
         
+    def getManipulatedDomainObject(self):
+        return self.getDomainObject()
+
 
 class AbstractInstanceView(AbstractClassView):
 
     def __init__(self, domainObjectKey=None, **kwds):
         super(AbstractInstanceView, self).__init__(**kwds)
-        self.domainObjectKey = domainObjectKey
+        if domainObjectKey:
+            self.domainObjectKey = domainObjectKey
         self.manipulator = None
 
     def setContext(self):
@@ -660,16 +743,22 @@ class AbstractInstanceView(AbstractClassView):
 
     def buildManipulator(self):
         manipulatorClass = self.getManipulatorClass()
+        if not issubclass(manipulatorClass, DomainObjectManipulator):
+            msg = "Manipulator class isn't a DomainObjectManipulator: %s" % (
+                manipulatorClass
+            )
+            raise Exception(msg)
         objectRegister   = self.getManipulatedObjectRegister()
         domainObject     = self.getManipulatedDomainObject()
         fieldNames       = self.getManipulatedFieldNames()
-        msg = "Building manipulator %s with fields names %s for position %s" % (
-            manipulatorClass.__name__,
-            fieldNames,
-            self.getViewPosition()
-        )
-        self.logger.debug(msg)
-        return manipulatorClass(objectRegister, domainObject, fieldNames)
+        if moddebug and self.debug:
+            msg = "Building manipulator %s for position %s" % (
+                manipulatorClass.__name__,
+                self.getViewPosition()
+            )
+            self.logger.debug(msg)
+            
+        return manipulatorClass(objectRegister, domainObject, fieldNames, self)
 
     def getManipulatedFieldNames(self):
         return []
@@ -683,7 +772,6 @@ class AbstractFormView(AbstractInstanceView):
     def __init__(self, **kwds):
         super(AbstractFormView, self).__init__(**kwds)
         self.form = None
-        self.requestParams = None
         self.formErrors = None
         
     def takeAction(self):
@@ -708,29 +796,16 @@ class AbstractFormView(AbstractInstanceView):
         isInitialSubmission = requestParams.has_key('initialise')
         return countParams and not isInitialSubmission
 
-    def getRequestParams(self):
-        if self.requestParams == None:
-            if self.request.POST:
-                self.requestParams = self.request.POST.copy()
-            elif self.request.GET:
-                self.requestParams = self.request.GET.copy()
-            else:
-                self.requestParams = self.getZeroRequestParams()
-        self.logger.debug("Request params: %s" % self.requestParams)
-        return self.requestParams
-
     def getFormErrors(self):
         if self.formErrors == None:
             if self.getRequestParams():
                 self.formErrors = self.getValidationErrors()
             else:
                 self.formErrors = self.getZeroFormErrors() 
-        self.logger.debug("Form errors: %s" % self.formErrors)
+        if moddebug and self.debug:
+            self.logger.debug("Form errors: %s" % self.formErrors)
         return self.formErrors
 
-    def getZeroRequestParams(self):
-        return {}
-
     def getZeroFormErrors(self):
         return {}
 
@@ -738,7 +813,8 @@ class AbstractFormView(AbstractInstanceView):
         manipulator = self.getManipulator()
         requestParams = self.getRequestParams()
         validationErrors = manipulator.getValidationErrors(requestParams)
-        self.logger.debug("Validation errors: %s" % validationErrors)
+        if moddebug and self.debug:
+            self.logger.debug("Validation errors: %s" % validationErrors)
         return validationErrors
             
     def getForm(self):
@@ -746,9 +822,6 @@ class AbstractFormView(AbstractInstanceView):
             self.form = self.buildForm()
         return self.form
 
-    def getManipulatedDomainObject(self):
-        return self.getDomainObject()
-
     def buildForm(self):
         manipulator = self.getManipulator()
         requestParams = self.getRequestParams()
@@ -779,7 +852,7 @@ class AbstractFormView(AbstractInstanceView):
         self.setRedirect(self.makePostManipulateLocation())
         
     def makePostManipulateLocation(self):
-        raise "Abstract method not implemented on class %s." % self.__class__.__name__
+        raise Exception("Abstract method not implemented on class %s." % self.__class__.__name__)
 
 
 class AbstractInputFormView(AbstractFormView):
@@ -813,6 +886,13 @@ class AbstractListView(AbstractClassView):
         })
 
 
+class AbstractPendingView(AbstractListView):
+
+    def getManipulatedObjectRegister(self):
+        r = super(AbstractPendingView, self).getManipulatedObjectRegister()
+        return r.getPending()
+        
+
 class AbstractSearchView(AbstractClassView):
 
     def __init__(self, startsWith='', **kwds):
@@ -842,7 +922,7 @@ class AbstractSearchView(AbstractClassView):
 
     def searchManipulatedRegister(self):
         if not self.domainClassName:
-            raise "No domainClassName on '%s' set." % self
+            raise Exception("No domainClassName on '%s' set." % self)
         commandName = self.domainClassName + 'List'
         commandKwds = {}
         commandKwds['userQuery'] = self.userQuery
@@ -857,6 +937,44 @@ class AbstractSearchView(AbstractClassView):
         return command.results
             
 
+class AbstractFindView(AbstractClassView):
+
+    def __init__(self, startsWith='', **kwds):
+        super(AbstractFindView, self).__init__(**kwds)
+        self.startsWith = startsWith
+        if not self.startsWith and self.request.POST:
+            self.requestParams = self.request.POST.copy()
+            if self.requestParams.has_key('startsWith'):
+                self.startsWith = self.requestParams['startsWith']
+
+    def setContext(self):
+        super(AbstractFindView, self).setContext()
+        if self.startsWith:
+            resultList = self.findDomainObjects()
+            self.context.update({
+                'isResultsRequest' : True,
+                'domainObjectList' : resultList,
+                'resultsCount'     : len(resultList),
+                'isResultSingular' : len(resultList) == 1,
+                'startsWith'       : self.startsWith,
+            })
+
+    def findDomainObjects(self):
+        if not self.domainClassName:
+            raise Exception("No domainClassName on '%s' set." % self)
+        commandName = self.domainClassName + 'List'
+        commandKwds = {}
+        commandKwds['startsWith'] = self.startsWith.capitalize()
+        if commandName in self.commands:
+            commandClass = self.commands[commandName]
+        else:
+            commandClass = self.commands['DomainObjectList']
+            commandKwds['typeName'] = self.domainClassName
+        command = commandClass(**commandKwds)
+        command.execute()
+        return command.results
+            
+
 class AbstractCreateView(AbstractInputFormView):
 
     def getInitialParams(self):
@@ -864,7 +982,8 @@ class AbstractCreateView(AbstractInputFormView):
         objectRegister = self.getManipulatedObjectRegister()
         domainClass = self.getDomainClass(objectRegister.typeName)
         for metaAttr in domainClass.meta.attributes:
-            if not (hasattr(metaAttr, 'default') and metaAttr.default):
+            #if not (hasattr(metaAttr, 'default') and metaAttr.default):
+            if not (hasattr(metaAttr, 'default')):
                 continue
             if metaAttr.isAssociateList:
                 initialParams.setlist(metaAttr.name, metaAttr.default)
@@ -879,22 +998,44 @@ class AbstractCreateView(AbstractInputFormView):
         super(AbstractCreateView, self).manipulateDomainObject()
         if not self.getFormErrors():
             manipulator = self.getManipulator()
-            manipulator.create(self.getRequestParams())
+            requestParams = self.getCreateParams()
+            manipulator.create(requestParams)
             if not manipulator.domainObject:
-                raise "Create manipulator did not produce an object."
+                raise Exception("Create manipulator did not produce an object.")
             self.setCreatedObject(manipulator.domainObject)
 
+    def getCreateParams(self):
+        return self.getRequestParams()
+
     def setCreatedObject(self, createdObject):
         self.domainObject = createdObject
     
 
+class AbstractImageView(AbstractInstanceView):
+
+    def getResponse(self):
+        self.content = self.getImageContent()
+        self.response = HttpResponse(self.content)
+        self.response.headers['Content-Type'] = self.getImageContentType()
+        return self.response
+
+    def getImageContent(self):
+        raise Exception("Method not implemented on %s" % self)
+        
+    def getImageContentType(self):
+        raise Exception("Method not implemented on %s" % self)
+        
+
 class AbstractReadView(AbstractInstanceView):
 
     def setContext(self):
         super(AbstractReadView, self).setContext()
+        (previousObject, nextObject) = self.getPreviousNextDomainObjects()
         self.context.update({
-            'domainObjectPersonalLabel' : self.getDomainObjectPersonalLabel(),
-            'domainObjectNamedValues'  : self.getDomainObjectAsNamedValues(),
+            'domainObjectPersonalLabel': self.getDomainObjectPersonalLabel(),
+            'domainObjectNamedValues': self.getDomainObjectAsNamedValues(),
+            'previousObject': previousObject,
+            'nextObject': nextObject,
         })
 
     def getDomainObjectAsNamedValues(self):
@@ -908,30 +1049,68 @@ class AbstractReadView(AbstractInstanceView):
         else:
             return ''
 
+    def getNextDomainObject(self):
+        domainObject = self.getManipulatedDomainObject()
+        objectRegister = self.getManipulatedObjectRegister()
+        objectList = objectRegister.getSortedList()
+        nextObject = objectRegister.getNextObject(
+            objectList, domainObject
+        )
+        return nextObject
+    
+    def getPreviousDomainObject(self):
+        domainObject = self.getManipulatedDomainObject()
+        objectRegister = self.getManipulatedObjectRegister()
+        objectList = objectRegister.getSortedList()
+        previousObject = objectRegister.getPreviousObject(
+            objectList, domainObject
+        )
+        return previousObject
+
+    def getPreviousNextDomainObjects(self):
+        domainObject = self.getManipulatedDomainObject()
+        objectRegister = self.getManipulatedObjectRegister()
+        objectList = objectRegister.getSortedList()
+        nextObject = objectRegister.getNextObject(
+            objectList, domainObject
+        )
+        previousObject = objectRegister.getPreviousObject(
+            objectList, domainObject
+        )
+        return (previousObject, nextObject)
+
 
 class AbstractUpdateView(AbstractInputFormView):
 
     def getInitialParams(self):
         domainObject = self.getManipulatedDomainObject()
-        domainObjectValueDict = domainObject.asDictValues()
-        initialParams = MultiValueDict()
-        for key in domainObjectValueDict:
-            item = domainObjectValueDict[key]
-            if hasattr(item, '__class__') and item.__class__.__name__ == 'list':
-                initialParams.setlist(key, item)
-            else:
-                if item == None:
-                    initialParams[key] = ''
-                else:
-                    initialParams[key] = item
-        return initialParams
+        return domainObject.asRequestParams()
         
     def manipulateDomainObject(self):
         super(AbstractUpdateView, self).manipulateDomainObject()
-        if not self.getFormErrors():
+        formErrors = self.getFormErrors()
+        if not formErrors:
             manipulator = self.getManipulator()
-            requestParams = self.getRequestParams()
-            manipulator.update(data=requestParams)
+            requestParams = self.getUpdateParams()
+            manipulator.update(requestParams)
+        else:
+            if self.debug:
+                self.logger.debug('Manipulator update not possible with form errors: %s' % (formErrors))
+
+    def getUpdateParams(self):
+        return self.getRequestParams()
+
+
+class AbstractApproveView(AbstractUpdateView):
+
+    def __init__(self, **kwds):
+        super(AbstractApproveView, self).__init__(**kwds)
+        self.formErrors = {}
+
+    def manipulateDomainObject(self):
+        super(AbstractApproveView, self).manipulateDomainObject()
+        domainObject = self.getManipulatedDomainObject()
+        domainObject.approve()
 
 
 class AbstractDeleteView(AbstractFormView):
